@@ -37,13 +37,13 @@
 static int get_scale_factor(H264Context *const h, int poc, int poc1, int i)
 {
     int poc0 = h->ref_list[0][i].poc;
-    int td = av_clip(poc1 - poc0, -128, 127);
+    int td = av_clip_int8(poc1 - poc0);
     if (td == 0 || h->ref_list[0][i].long_ref) {
         return 256;
     } else {
-        int tb = av_clip(poc - poc0, -128, 127);
+        int tb = av_clip_int8(poc - poc0);
         int tx = (16384 + (FFABS(td) >> 1)) / td;
-        return av_clip((tb * tx + 32) >> 6, -1024, 1023);
+        return av_clip_intp2((tb * tx + 32) >> 6, 10);
     }
 }
 
@@ -600,7 +600,7 @@ single_col:
 
                 {
                     const int16_t *mv_col = l1mv[x8 * 3 + y8 * b4_stride];
-                    int my_col            = (mv_col[1] * (1 << y_shift)) / 2;
+                    int my_col            = (mv_col[1] << y_shift) / 2;
                     int mx                = (scale * mv_col[0] + 128) >> 8;
                     int my                = (scale * my_col    + 128) >> 8;
                     fill_rectangle(&h->mv_cache[0][scan8[i8 * 4]], 2, 2, 8,
