@@ -246,7 +246,7 @@ static av_cold void uninit(AVFilterContext *ctx)
     int i;
 
     for (i = 0; i < s->nb_delays; i++)
-        av_free(s->chandelay[i].samples);
+        av_freep(&s->chandelay[i].samples);
     av_freep(&s->chandelay);
 }
 
