@@ -47,6 +47,7 @@ static const char *const var_names[] = {
     "h",              ///< height of the rendered box
     "w",              ///< width  of the rendered box
     "t",
+    "max",
     NULL
 };
 
@@ -63,10 +64,11 @@ enum var_name {
     VAR_H,
     VAR_W,
     VAR_T,
+    VAR_MAX,
     VARS_NB
 };
 
-typedef struct {
+typedef struct DrawBoxContext {
     const AVClass *class;
     int x, y, w, h;
     int thickness;
@@ -142,30 +144,35 @@ static int config_input(AVFilterLink *inlink)
 
     for (i = 0; i <= NUM_EXPR_EVALS; i++) {
         /* evaluate expressions, fail on last iteration */
+        var_values[VAR_MAX] = inlink->w;
         if ((ret = av_expr_parse_and_eval(&res, (expr = s->x_expr),
                                           var_names, var_values,
                                           NULL, NULL, NULL, NULL, NULL, 0, ctx)) < 0 && i == NUM_EXPR_EVALS)
             goto fail;
         s->x = var_values[VAR_X] = res;
 
+        var_values[VAR_MAX] = inlink->h;
         if ((ret = av_expr_parse_and_eval(&res, (expr = s->y_expr),
                                           var_names, var_values,
                                           NULL, NULL, NULL, NULL, NULL, 0, ctx)) < 0 && i == NUM_EXPR_EVALS)
             goto fail;
         s->y = var_values[VAR_Y] = res;
 
+        var_values[VAR_MAX] = inlink->w - s->x;
         if ((ret = av_expr_parse_and_eval(&res, (expr = s->w_expr),
                                           var_names, var_values,
                                           NULL, NULL, NULL, NULL, NULL, 0, ctx)) < 0 && i == NUM_EXPR_EVALS)
             goto fail;
         s->w = var_values[VAR_W] = res;
 
+        var_values[VAR_MAX] = inlink->h - s->y;
         if ((ret = av_expr_parse_and_eval(&res, (expr = s->h_expr),
                                           var_names, var_values,
                                           NULL, NULL, NULL, NULL, NULL, 0, ctx)) < 0 && i == NUM_EXPR_EVALS)
             goto fail;
         s->h = var_values[VAR_H] = res;
 
+        var_values[VAR_MAX] = INT_MAX;
         if ((ret = av_expr_parse_and_eval(&res, (expr = s->t_expr),
                                           var_names, var_values,
                                           NULL, NULL, NULL, NULL, NULL, 0, ctx)) < 0 && i == NUM_EXPR_EVALS)
@@ -271,7 +278,7 @@ static const AVFilterPad drawbox_outputs[] = {
     { NULL }
 };
 
-AVFilter avfilter_vf_drawbox = {
+AVFilter ff_vf_drawbox = {
     .name          = "drawbox",
     .description   = NULL_IF_CONFIG_SMALL("Draw a colored box on the input video."),
     .priv_size     = sizeof(DrawBoxContext),
@@ -377,7 +384,7 @@ static const AVFilterPad drawgrid_outputs[] = {
     { NULL }
 };
 
-AVFilter avfilter_vf_drawgrid = {
+AVFilter ff_vf_drawgrid = {
     .name          = "drawgrid",
     .description   = NULL_IF_CONFIG_SMALL("Draw a colored grid on the input video."),
     .priv_size     = sizeof(DrawBoxContext),
